# -*- coding: utf-8 -*-

import os
import datetime
import json
import time
import re

import click
import dateparser

from em_tasks.utils.price_calculator import PriceCalculator
from em_tasks.utils.coupang_uploadable_products_filter import CoupangUploadableProductsFilter

from em_celery import logger, get_config, init_db, get_product_service


@click.command('Filter coupang uploable products.')
@click.option('-l', '--location_id', required=True, type=str, help='Location ID.')
@click.option('-o', '--output_path', type=str, required=True)
def format_products(output_path, location_id):
  init_db()
  product_service = get_product_service()

  price_rules = {
    'roi': 0.3,
    'ad_cost': 5,
    'transfer_cost': 12
  }

  cfg = get_config()
  price_rules_cfg = cfg.get('price.rules.coupang', dict())
  for k, v in price_rules_cfg.items():
    price_rules[k] = round(float(v), 2)
  price_calculator = PriceCalculator(price_rules)

  output_path = os.path.abspath(os.path.expanduser(output_path))
  output_dir = os.path.dirname(output_path)
  if not os.path.isdir(output_dir):
    os.makedirs(output_dir)

  coupang_formatter = CoupangProductFormatter(output_path, product_service, price_calculator, location_id)
  coupang_formatter.run()


class CoupangProductFormatter():
  def __init__(self, output_path, product_service, price_calculator, location_id):
    self.output_path = output_path
    self.product_service = product_service
    self.price_calc = price_calculator
    self.location_id = location_id
    self.indice_name = 'coupang_products'
    self.output_fh = open(self.output_path, 'w+', encoding='utf-8', errors='ignore')
    self.products_filter = CoupangUploadableProductsFilter(self.product_service)
    self.filter_patterns = [
      r'(.*?Food.*?)',
      r'(.*?Juice.*?)',
      r'(.*?Fresh.*?)',
      r'(.*?Drink.*?)',
      r'(.*?Refrigerate.*?)',
      r'(.*?Grill.*?)',
      r'(.*?Meat.*?)',
      r'(.*?Fruit.*?)',
      r'(.*?Beef.*?)',
      r'(.*?Pork.*?)',
      r'(.*?Chicken.*?)',
      r'(.*?mushroom.*?)',
      r'(.*?blueberry.*?)',
      r'(.*?Broccoli.*?)',
      r'(.*?Ginger.*?)',
      r'(.*?Tomato.*?)',
      r'(.*?Sunflower.*?)',
      r'(.*?pepper.*?)',
      r'(.*?Hanspoon.*?)',
      r'(.*?Orange.*?)',
      r'(.*?Pumpkin.*?)',
      r'(.*?onion.*?)',
      r'(.*?Potato.*?)',
      r'(.*?Ginseng.*?)',
      r'(.*?tofu.*?)',
      r'(.*?Primrose Oil.*?)',
      r'(.*?California Gold Nutrition.*?)',
      r'(.*?American Health Royal.*?)',
      r'(.*?Nutritional.*?)',
      r'(.*?Hongseong.*?)',
      r'(.*?belt massager.*?)',
      r'(.*?frozen.*?)',
      r'(.*?kg.*?)',
      r'(.*?Large.*?)',
      r'(.*?Heavy.*?)',
      r'(.*?HOMETLOVE.*?)',
      r'(.*?Ballet Bar.*?)',
    ]

  def run(self):
    batch_size = 500
    product_ids = []
    try:
      for pid, prod in self.products_filter.filter():
        product = self.format_product(prod)
        if not product:
          continue

        self.output_fh.write("\n{}".format(json.dumps(product, ensure_ascii=False)))
    finally:
      self.output_fh.close()

  def format_product(self, product):
    src_product_price = product.get('price', None)
    if not src_product_price:
      return

    src_product_price = round(float(src_product_price), 2)

    currency = product.get('currency', None) or 'USD'
    src_offer = {'price': src_product_price, 'currency': currency}
    offer = self.price_calc.calc_offer(src_offer)
    if not offer or not offer['quantity']:
      return

    title = product['title_en']
    for pattern in self.filter_patterns:
      title = re.sub(pattern, '', title, flags=re.IGNORECASE)

    title = title.strip()
    if not title:
      return

    product['title_en'] = title
    product['price'] = offer['price']
    product['quantity'] = offer['quantity']
    product['sku'] = 'X03-{}'.format(product['product_id'])
    product['description'] = product.get('description_en', None) or product['description']
    product['location_id'] = self.location_id
    product.pop('seller_id', None)
    options = product.get('options', None) or []
    for option in options:
      name = option.get('name_en', option.get('option_name_en', None))
      if not name:
        name = option.get('name', option.get('option_name', None))

      option['name'] = name

    if product.get('variants', []):
      for v in product['variants']:
        v_price = v.get('price', None) or 0
        if v_price:
          v_price = round(float(v_price), 2)
        src_offer = {'price': v_price, 'currency': currency}
        offer = self.price_calc.calc_offer(src_offer)
        if not offer or not offer['quantity']:
          v['quantity'] = 0
          continue

        v['price'] = offer['price']
        v['quantity'] = offer['quantity']

        option_values = v.get('option_values', [])
        if not option_values:
          continue

        invalid_variant = False
        for ov in option_values:
          option_name = ov.get('option_name_en', None)
          if not option_name:
            option_name = ov.get('option_name', None)
          ov['option_name'] = option_name

          ov['option_value'] = ov.get('option_value_en', ov.get('option_value', None))

    product['categories'] = 'Korea'

    return product


if __name__ == '__main__':
  format_products()
