# -*- coding: utf-8 -*-

import os
import sys
import datetime
import time
import json
try:
  from urllib.parse import urlparse
  from urllib.parse import parse_qs
except ImportError:
  from urlparse import urlparse
  from urlparse import parse_qs

import click
import redis
import dateparser

from em_tasks.store.store_util import StoreUtil
from em_tasks.spree.product_util import ProductUtil

from em_celery import logger, init_db, get_product_service


class EbayOffersUpdateTaskSender():
  def __init__(self, store_code, marketplace, redis_uri, start_urls_key, qps, ttl=3):
    store = StoreUtil.get_store_by_code(store_code)
    credential = json.loads(store.api_credential)
    self.data_source = ProductUtil(credential['endpoint'], credential['api_key'], credential.get('api_version', 'v1'))
    self.marketplace = marketplace.upper()
    self.r = redis.Redis.from_url(redis_uri)
    self.start_urls_key = start_urls_key
    self.qps = qps
    self.last_send_time = None
    self.product_service = get_product_service()
    self.indice_name = 'ebay_{}_products'.format(marketplace.lower())
    today = datetime.date.today()
    self.expire_date = today - datetime.timedelta(days=ttl)
    self.max_tasks_cnt = 5000

  def run(self):
    cnt = self.tasks_cnt()
    if cnt > self.max_tasks_cnt:
      logger.info('[TasksProcessing] %s', cnt)
      return

    if cnt > 0:
      self.expire_date = datetime.date.today() - datetime.timedelta(days=5)

    product_ids = []
    batch_size = 500
    try:
      for prod in self.data_source.get_ebay_products(self.marketplace):
        product_ids.append(prod['source_product_id'])
        if len(product_ids) < batch_size:
          continue

        self.process_products(product_ids)
        product_ids = []
    except (KeyboardInterrupt, SystemExit):
      print('Exiting, please wait...')
    finally:
      if product_ids:
        self.process_products(product_ids)
        product_ids = []

  def process_products(self, product_ids):
    pids = dict()
    for pid in product_ids:
      pids[pid] = 'https://www.ebay.com/itm/{}'.format(pid)

    products = False
    max_retries = 3
    while max_retries > 0:
      try:
        products = self.product_service.search_products(self.indice_name, product_ids)
        if not isinstance(products, dict):
          logger.info('[ProductService] Temporary unavailable! Wait for 10 seconds to retry.')
          max_retries -= 1
          time.sleep(10)
          continue

        break
      except Exception as e:
        logger.exception(e)
        max_retries -= 1
        time.sleep(3)

    if not products:
      return

    for pid, product in products.items():
      existence = product.get('existence', True)
      if existence and (not product.get('title', None) or not product.get('images', None)):
        continue

      update_date_s = product.get('date', product.get('time', None))
      if not update_date_s:
        continue

      try:
        update_date = dateparser.parse(update_date_s).date()
        # Not expired, no need to update
        if update_date > self.expire_date:
          pids.pop(pid, None)
      except:
        continue

    urls = list(pids.values())
    for product_url in urls:
      if self.last_send_time:
        wait_time = 1 / self.qps - (time.time() - self.last_send_time)
        if wait_time > 0:
          print("Waiting %.3fs to send next message" % wait_time)
          time.sleep(wait_time)

      self.last_send_time = time.time()

      try:
        self.r.lpush(self.start_urls_key, json.dumps({'url': product_url}))
        logger.debug('Added %s', product_url)
      except:
        pass

  def tasks_cnt(self):
    cnt = 0
    try:
      cnt = self.r.llen(self.start_urls_key)
    except:
      pass

    return cnt
